/**
 * Copyright (C) 2010-2011 enStratusNetworks LLC
 *
 * ====================================================================
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * ====================================================================
 */

package org.dasein.cloud.jclouds.gogrid;

import static org.jclouds.concurrent.MoreExecutors.sameThreadExecutor;

import org.apache.log4j.Logger;
import org.dasein.cloud.AbstractCloud;
import org.dasein.cloud.ProviderContext;
import org.dasein.cloud.compute.Architecture;
import org.dasein.cloud.compute.Platform;
import org.dasein.cloud.jclouds.gogrid.compute.GoGridComputeServices;
import org.dasein.cloud.jclouds.gogrid.network.GoGridNetworkServices;
import org.jclouds.concurrent.config.ExecutorServiceModule;
import org.jclouds.gogrid.GoGridAsyncClient;
import org.jclouds.gogrid.GoGridClient;
import org.jclouds.gogrid.domain.Option;
import org.jclouds.logging.log4j.config.Log4JLoggingModule;
import org.jclouds.rest.RestContext;
import org.jclouds.rest.RestContextFactory;

import com.google.common.collect.ImmutableSet;
import com.google.inject.Module;

public class GoGrid extends AbstractCloud {
    static private final Logger logger = Logger.getLogger(GoGrid.class);
    
    public GoGrid() { }
    
    @Override
    public String getCloudName() {
        return "GoGrid";
    }

    public RestContext<GoGridClient,GoGridAsyncClient> getCloudClient() {
        ProviderContext ctx = getContext();
        String user = new String(ctx.getAccessPublic());
        String key = new String(ctx.getAccessPrivate());
        
        return (new RestContextFactory()).createContext("gogrid", user, key, ImmutableSet.<Module> of(new Log4JLoggingModule(), new ExecutorServiceModule(sameThreadExecutor(), sameThreadExecutor())));
    }
    
    @Override
    public GoGridComputeServices getComputeServices() {
        return new GoGridComputeServices(this);
    }
    
    @Override
    public GoGridDc getDataCenterServices() {
        return new GoGridDc(this);
    }

    @Override
    public GoGridNetworkServices getNetworkServices() {
        return new GoGridNetworkServices(this);
    }
    
    @Override
    public String getProviderName() {
        return "GoGrid";
    }
    
    public Architecture toArchitecture(Option os, String description) {
        Architecture a = toArchitecture(os.getName());
        
        if( a == null && description != null ) {
            a = toArchitecture(description);
        }
        return (a == null ? Architecture.I64 : a);
    }
    
    private Architecture toArchitecture(String name) {
        if( name.contains("_32_") ) {
            return Architecture.I32;
        }
        else if( name.contains("_64_") ) {
            return Architecture.I64;
        }
        else if( name.contains("64") ) {
            return Architecture.I64;
        }
        else if( name.contains("32") ) {
            return Architecture.I32;
        }
        return null;
    }
    
    public Platform toPlatform(Option original) {
        if( original == null ) {
            return Platform.UNKNOWN;
        }
        return Platform.guess(original.getName());
    }
    
    @Override
    public String testContext() {
        try {
            if( !getComputeServices().getVirtualMachineSupport().isSubscribed() ) {
                return null;
            }
            return getContext().getAccountNumber();
        }
        catch( Throwable t ) {
            logger.warn("Failed to test GoGrid connection context: " + t.getMessage());
            if( logger.isDebugEnabled() ) {
                t.printStackTrace();
            }
            return null;
        }
    }
}
